/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.paramchooser;

import java.util.Collection;
import java.util.TreeMap;

import org.appspy.admin.client.settings.form.ApplicationForm;
import org.appspy.admin.client.settings.form.ApplicationVersionForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;
import com.gwtext.client.widgets.tree.event.TreePanelListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class VersionChooser extends AbstractParamChooser {
	
	protected TreePanel mTreePanel = new TreePanel();

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		mTreePanel.setTitle("Versions");
		mTreePanel.setAutoScroll(true);
		mTreePanel.setBorder(false);
		mTreePanel.setIconCls("settings-menu-icon");
		mTreePanel.setEnableDD(false);
		TreeNode root = new TreeNode("Versions");
		root.setExpanded(true);
		mTreePanel.setRootNode(root);
		mTreePanel.setRootVisible(false);
		
		getAdminModule().getAdminModuleService().getVersions(new AsyncCallback<Collection<ApplicationVersionForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<ApplicationVersionForm> result) {
				TreeMap<String, ApplicationForm> orgMap = new TreeMap<String, ApplicationForm>();
				TreeMap<String, ApplicationVersionForm> versionMap = new TreeMap<String, ApplicationVersionForm>();
				for (ApplicationVersionForm version : result) {
					orgMap.put(version.getApplication().getName(), version.getApplication());
					versionMap.put(version.getName(), version);
				}
				for (ApplicationForm app : orgMap.values()) {
					final TreeNode appTreeNode = new TreeNode(app.getName());
					appTreeNode.setUserObject(app);
					appTreeNode.setIconCls("settings-menu-icon");
					mTreePanel.getRootNode().appendChild(appTreeNode);
					for (ApplicationVersionForm version : versionMap.values()) {
						if (version.getApplication() == app)
						{
							final TreeNode versionTreeNode = new TreeNode(version.getName());
							versionTreeNode.setUserObject(version);
							versionTreeNode.setIconCls("settings-menu-icon");
							appTreeNode.appendChild(versionTreeNode);
							if (version.getId().toString().equals(getValue()))
							{
//								appTreeNode.setExpanded(true);
								versionTreeNode.ensureVisible();
								versionTreeNode.select();
							}
						}
					}
				}
			}});
		
		mTreePanel.addListener(new TreePanelListenerAdapter(){

			@Override
			public void onClick(TreeNode node, EventObject e) {
				try
				{
					if (node.getUserObject() instanceof ApplicationVersionForm)
					{
						ApplicationVersionForm version = (ApplicationVersionForm) node.getUserObject();
						setValue(version.getId().toString());
					}
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		return mTreePanel;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#updateTextField(com.gwtext.client.widgets.form.TextField)
	 */
	public void updateTextField(final TextField textField) {
		getAdminModule().getAdminModuleService().getVersions(new AsyncCallback<Collection<ApplicationVersionForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<ApplicationVersionForm> result) {
				for (ApplicationVersionForm version : result) {
					if (version.getId().toString().equals(getValue()))
					{
						textField.setValue(version.getName());
					}
				}
			}});
	}
	
	
}
