/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.settings;

import java.util.Arrays;
import java.util.Collection;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.settings.form.ApplicationForm;
import org.appspy.admin.client.settings.form.ApplicationVersionForm;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListVersionPanel extends AbstractPanel {

	public static final String TAB_ID = "ListVersionPanel";
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new StringFieldDef("appName"),
			new IntegerFieldDef("appId")});
	
	protected RecordDef mAppRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("code"), 
			new StringFieldDef("name")});
	
	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	protected Store mAppStore = null;
	
	public ListVersionPanel(AdminModule adminModule) {
		
		super(adminModule);
		
		setTitle("Versions");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig appNameColumnConfig = new ColumnConfig("APPLICATION", "appName", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				appNameColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(2);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		mAppStore = new Store(mAppRecordDef);
		
		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New version");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(250);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

				final ApplicationVersionForm version = new ApplicationVersionForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						version,
						null, 
						"", 
						"",
						null});
				
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));

		        final ComboBox appComboBox = new ComboBox("Application", "appId");
		        formPanel.add(appComboBox, new AnchorLayoutData("100%"));
		        appComboBox.setStore(mAppStore);
		        appComboBox.setValueField("id");
		        appComboBox.setDisplayField("name");
		        appComboBox.setMode(ComboBox.LOCAL);
		        appComboBox.setForceSelection(true);
		        appComboBox.setDisableKeyFilter(true);
		        appComboBox.setLazyRender(true);
		        appComboBox.setTriggerAction(ComboBox.ALL);

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);
				
				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						version.setName(record.getAsString("name"));
						ApplicationForm app = (ApplicationForm) mAppStore.query("id", appComboBox.getValueAsString())[0].getAsObject("object");
						record.set("appName", app.getName());
						version.setApplication(app);
						final AsyncCallback<ApplicationVersionForm> callback = new AsyncCallback<ApplicationVersionForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Technical problem", "Unable to create version");
							}

							public void onSuccess(ApplicationVersionForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								window.close();
							}
						};
						
						getAdminModule().getAdminModuleService().saveVersion(version, callback);
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Be carefull before deleting a version. Do you confirm ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											ApplicationVersionForm version = (ApplicationVersionForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteVersion(version, new AsyncCallback<ApplicationForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete version", caught.getLocalizedMessage());
												}

												public void onSuccess(ApplicationForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);  

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(50);
        centerLayoutData.setMinWidth(300);
        
        mGridPanel.setWidth(400);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(250);
        mFormPanel.setLabelWidth(80);
        
        mFormPanel.setTitle("Version Properties");
        mFormPanel.setDisabled(true);
        
        final TextField nameTextField = new TextField("Name", "name", 120);
        mFormPanel.add(nameTextField, new AnchorLayoutData("100%"));
        
        final ComboBox orgComboBox = new ComboBox("Application", "appId", 120);
        mFormPanel.add(orgComboBox, new AnchorLayoutData("100%"));
        orgComboBox.setStore(mAppStore);
        orgComboBox.setValueField("id");
        orgComboBox.setDisplayField("name");
        orgComboBox.setMode(ComboBox.LOCAL);
        orgComboBox.setForceSelection(true);
        orgComboBox.setDisableKeyFilter(true);
        orgComboBox.setLazyRender(true);
        orgComboBox.setTriggerAction(ComboBox.ALL);
        
        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);

        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<ApplicationVersionForm> callback = new AsyncCallback<ApplicationVersionForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Technical problem", "Unable to update version");
					}

					public void onSuccess(ApplicationVersionForm result) {
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final ApplicationVersionForm version = (ApplicationVersionForm) record.getAsObject("object");
				version.setName(record.getAsString("name"));
				ApplicationForm app = (ApplicationForm) mAppStore.query("id", orgComboBox.getValue())[0].getAsObject("object");
				record.set("appName", app.getName());
				version.setApplication(app);
				if (Arrays.asList(record.getModifiedFields()).contains("name"))
				{
					MessageBox.ConfirmCallback confirmCallback = new MessageBox.ConfirmCallback(){
						public void execute(String btnID) {
							if (btnID.equals("yes")) {
								getAdminModule().getAdminModuleService().updateVersion(version, callback);
							}
							else
							{
								record.reject();
								mFormPanel.getForm().loadRecord(record);
							}
						}
					};
					MessageBox.confirm("Confirm", "Be carefull before changing the name of a version. Do you confirm ?",
							confirmCallback);
				}
				else
				{
					getAdminModule().getAdminModuleService().updateVersion(version, callback);
				}
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(220);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<ApplicationVersionForm>> versionCallback = new AsyncCallback<Collection<ApplicationVersionForm>>() {
			public void onSuccess(Collection<ApplicationVersionForm> applications) {

				mStore.removeAll();
				mStore.commitChanges();
				for (ApplicationVersionForm version : applications) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							version,
							version.getId(), 
							version.getName(),
							version.getApplication() == null ? null : version.getApplication().getName(),
							version.getApplication() == null ? null : version.getApplication().getId()}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		AsyncCallback<Collection<ApplicationForm>> appCallback = new AsyncCallback<Collection<ApplicationForm>>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(Collection<ApplicationForm> result) {
				
				mAppStore.removeAll();
				for (ApplicationForm app : result) {
					mAppStore.add(mAppRecordDef.createRecord(new Object[]{
							app,
							app.getId(), 
							app.getCode(), 
							app.getName()}));
				}
				mAppStore.commitChanges();
				getAdminModule().getAdminModuleService().getVersions(versionCallback);
			}};
		
		getAdminModule().getAdminModuleService().getApplications(appCallback);
	}
}
