/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.settings;

import org.appspy.admin.client.AbstractTreePanel;
import org.appspy.admin.client.settings.action.OpenListApplicationPanelAction;
import org.appspy.admin.client.settings.action.OpenListEnvironmentPanelAction;
import org.appspy.admin.client.settings.action.OpenListHostPanelAction;
import org.appspy.admin.client.settings.action.OpenListOrganizationPanelAction;
import org.appspy.admin.client.settings.action.OpenListVersionPanelAction;

import com.gwtext.client.core.EventObject;
import com.gwtext.client.data.Node;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.event.TreeNodeListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class SettingsMenuTreePanel extends AbstractTreePanel {

	public SettingsMenuTreePanel() {

		TreeNode root = new TreeNode("Settings");
		root.setExpanded(true);

		TreeNode orgTreeNode = new TreeNode("Organizations");
		orgTreeNode.setTooltip("Manage the organizations");
		orgTreeNode.setIconCls("settings-item-icon");
		root.appendChild(orgTreeNode);
		orgTreeNode.addListener(new TreeNodeListenerAdapter() {

			@Override
			public void onClick(Node node, EventObject e) {
				
				new OpenListOrganizationPanelAction(getAdminModule()).execute();
			}});

		TreeNode envTreeNode = new TreeNode("Environments");
		envTreeNode.setTooltip("Manage the environments");
		envTreeNode.setIconCls("settings-item-icon");
		root.appendChild(envTreeNode);
		envTreeNode.addListener(new TreeNodeListenerAdapter() {

			@Override
			public void onClick(Node node, EventObject e) {
				
				new OpenListEnvironmentPanelAction(getAdminModule()).execute();
			}});

		TreeNode hostTreeNode = new TreeNode("Hosts");
		hostTreeNode.setTooltip("Manage the hosts");
		hostTreeNode.setIconCls("settings-item-icon");
		root.appendChild(hostTreeNode);
		hostTreeNode.addListener(new TreeNodeListenerAdapter() {

			@Override
			public void onClick(Node node, EventObject e) {
				
				new OpenListHostPanelAction(getAdminModule()).execute();
			}});

		TreeNode appTreeNode = new TreeNode("Applications");
		appTreeNode.setTooltip("Manage the applications");
		appTreeNode.setIconCls("settings-item-icon");
		root.appendChild(appTreeNode);
		appTreeNode.addListener(new TreeNodeListenerAdapter() {

			@Override
			public void onClick(Node node, EventObject e) {
				
				new OpenListApplicationPanelAction(getAdminModule()).execute();
			}});

		TreeNode versionTreeNode = new TreeNode("Application Versions");
		versionTreeNode.setTooltip("Manage the versions of the applications");
		versionTreeNode.setIconCls("settings-item-icon");
		root.appendChild(versionTreeNode);
		versionTreeNode.addListener(new TreeNodeListenerAdapter() {

			@Override
			public void onClick(Node node, EventObject e) {
				
				new OpenListVersionPanelAction(getAdminModule()).execute();
			}});

		setTitle("Settings");
		setWidth(200);
		setHeight(400);
		setEnableDD(true);
		setRootVisible(false);
		setRootNode(root);
	}
}
